var pad = function(i) {
	return (i < 10 ? '0' + i : i);
};

Ext.chart.series.Bar.prototype.drawSeries = function() {
	var me = this, chart = me.chart, store = chart.getChartStore(), surface = chart.surface, animate = chart.animate, stacked = me.stacked, column = me.column, chartAxes = chart.axes, boundAxes = me
			.getAxesForXAndYFields(), boundXAxis = boundAxes.xAxis, boundYAxis = boundAxes.yAxis, enableShadows = chart.shadow, shadowGroups = me.shadowGroups, shadowGroupsLn = shadowGroups.length, group = me.group, seriesStyle = me.seriesStyle, items, ln, i, j, baseAttrs, sprite, rendererAttributes, shadowIndex, shadowGroup, bounds, endSeriesStyle, barAttr, attrs, anim;

	if (!store || !store.getCount() || me.seriesIsHidden) {
		me.hide();
		me.items = [];
		return;
	}

	// fill colors are taken from the colors array.
	endSeriesStyle = Ext.apply({}, this.style, seriesStyle);
	delete endSeriesStyle.fill;
	delete endSeriesStyle.x;
	delete endSeriesStyle.y;
	delete endSeriesStyle.width;
	delete endSeriesStyle.height;

	me.unHighlightItem();
	me.cleanHighlights();

	me.boundColumn = (boundXAxis && Ext.Array.contains(me.axis, boundXAxis) && chartAxes.get(boundXAxis) && chartAxes.get(boundXAxis).isNumericAxis);

	me.getPaths();
	bounds = me.bounds;
	items = me.items;

	baseAttrs = column ? {
		y : bounds.zero,
		height : 0
	} : {
		x : bounds.zero,
		width : 0
	};
	ln = items.length;

	// Create new or reuse sprites and animate/display
	for (i = 0; i < ln; i++) {
		sprite = group.getAt(i);
		barAttr = items[i].attr;

		if (enableShadows) {
			items[i].shadows = me.renderShadows(i, barAttr, baseAttrs, bounds);
		}

		// Create a new sprite if needed (no height)
		if (!sprite) {
			attrs = Ext.apply({}, baseAttrs, barAttr);
			attrs = Ext.apply(attrs, endSeriesStyle || {});
			sprite = surface.add(Ext.apply({}, {
				type : 'path',
				group : group
			}, attrs));
		}
		if (animate) {
			rendererAttributes = me.renderer(sprite, store.getAt(i), barAttr, i, store);
			sprite._to = rendererAttributes;
			anim = me.onAnimate(sprite, {
				to : Ext.apply(rendererAttributes, endSeriesStyle)
			});
			if (enableShadows && stacked && (i % bounds.barsLen === 0)) {
				j = i / bounds.barsLen;
				for (shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
					anim.on('afteranimate', function() {
						this.show(true);
					}, shadowGroups[shadowIndex].getAt(j));
				}
			}
		} else {
			rendererAttributes = me.renderer(sprite, store.getAt(i), Ext.apply(barAttr, {
				hidden : false
			}), i, store);
			sprite.setAttributes(Ext.apply(rendererAttributes, endSeriesStyle), true);
			sprite.setStyle('stroke-width',1);
		}
		items[i].sprite = sprite;
	}

	// Hide unused sprites
	ln = group.getCount();
	for (j = i; j < ln; j++) {
		group.getAt(j).hide(true);
	}

	if (me.stacked) {
		// If stacked, we have only store.getCount() shadows.
		i = store.getCount();
	}

	// Hide unused shadows
	if (enableShadows) {
		for (shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
			shadowGroup = shadowGroups[shadowIndex];
			ln = shadowGroup.getCount();
			for (j = i; j < ln; j++) {
				shadowGroup.getAt(j).hide(true);
			}
		}
	}
	me.renderLabels();
};

Ext.chart.series.Bar.prototype.getPaths = function() {
	var me = this, chart = me.chart, store = chart.getChartStore(), data = store.data.items, i, total, record, bounds = me.bounds = me.getBounds(), items = me.items = [], yFields = Ext
			.isArray(me.yField) ? me.yField : [ me.yField ], gutter = me.gutter / 100, groupGutter = me.groupGutter / 100, animate = chart.animate, column = me.column, group = me.group, enableShadows = chart.shadow, shadowGroups = me.shadowGroups, shadowAttributes = me.shadowAttributes, shadowGroupsLn = shadowGroups.length, bbox = bounds.bbox, barWidth = bounds.barWidth, shrunkBarWidth = bounds.shrunkBarWidth, padding = me
			.getPadding(), stacked = me.stacked, barsLen = bounds.barsLen, colors = me.colorArrayStyle, colorLength = colors && colors.length || 0, themeIndex = me.themeIdx, reverse = me.reverse, math = Math, mmax = math.max, mmin = math.min, mabs = math.abs, j, yValue, height, totalDim, totalNegDim, bottom, top, hasShadow, barAttr, attrs, counter, totalPositiveValues, totalNegativeValues, shadowIndex, shadow, sprite, offset, floorY, idx, itemIdx, xPos, yPos, width, usedWidth, barIdx;

	for (i = 0, total = data.length; i < total; i++) {
		record = data[i];
		bottom = bounds.zero;
		top = bounds.zero;
		totalDim = 0;
		totalNegDim = 0;
		totalPositiveValues = totalNegativeValues = 0;
		hasShadow = false;
		usedWidth = 0;

		for (j = 0, counter = 0; j < barsLen; j++) {
			// Excluded series
			if (me.__excludes && me.__excludes[j]) {
				continue;
			}
			yValue = record.get(bounds.bars[j]);
			if (yValue >= 0) {
				totalPositiveValues += yValue;
			} else {
				totalNegativeValues += yValue;
			}
			height = Math.round((yValue - mmax(bounds.minY, 0)) * bounds.scale);
			idx = themeIndex + (barsLen > 1 ? j : 0);
			barAttr = {
				fill : colors[idx % colorLength]				
			};

			var typ = record.get('typ' + j);
			var iHeight = -1;
			switch (typ) {
			case OperationModes.transparent:
				barAttr.fill = '#7575F7';
				barAttr.opacity= 1;
                iHeight = 3;
				break;
			case OperationModes.nothing: // not active, placeholder
				// barAttr.opacity = ;
				barAttr.fill = '#888888';
				iHeight = 3;
				break;
			case OperationModes.Off: // comfort
				barAttr.fill = '#ff0000';
				iHeight = 20;
				break;
			case OperationModes.Comfort: // comfort
				barAttr.fill = '#ff0000';
				iHeight = 20;
				break;
			case OperationModes.Eco: // eco
				barAttr.fill = '#00ff00';
				iHeight = 20;
				break;
			case OperationModes.NotUsed: // not used
                barAttr.fill = '#04048A';
				iHeight = 20;
				break;
			case OperationModes.Night: // night
				barAttr.fill = '#222222';
				iHeight = 20;
				break;
			
			}

			if (column) {
				idx = reverse ? (total - i - 1) : i;
				barIdx = reverse ? (barsLen - counter - 1) : counter;

				if (me.boundColumn) {
					xPos = bounds.barsLoc[idx];
				} else if (me.configuredColumnGirth && bounds.barsLoc.length) {
					xPos = bounds.barsLoc[idx] + barIdx * bounds.groupBarWidth * (1 + groupGutter) * !stacked;

				} else {
					xPos = bbox.x + padding.left + (barWidth - shrunkBarWidth) * 0.5 + idx * barWidth * (1 + gutter) + barIdx * bounds.groupBarWidth * (1 + groupGutter) * !stacked;
				}

				Ext.apply(barAttr, {
					height : height,
					width : mmax(bounds.groupBarWidth, 0),
					x : xPos,
					y : bottom - height
				});
			} else {
				// draw in reverse order
				offset = (total - 1) - i;
				width = height + (bottom == bounds.zero);
				xPos = bottom + (bottom != bounds.zero);

				if (reverse) {
					// Subtract 1 for the first item
					xPos = bounds.zero + bbox.width - width - (usedWidth === 0 ? 1 : 0);
					if (stacked) {
						xPos -= usedWidth;
						usedWidth += width;
					}
				}

				if (me.configuredColumnGirth && bounds.barsLoc.length) {
					yPos = bounds.barsLoc[i] + counter * bounds.groupBarWidth * (1 + groupGutter) * !stacked;
				} else {
					yPos = bbox.y + padding.top + (barWidth - shrunkBarWidth) * 0.5 + offset * barWidth * (1 + gutter) + counter * bounds.groupBarWidth * (1 + groupGutter) * !stacked + 1;
				}
				var deltaY = 0;
				if (iHeight == -1) {
					iHeight = mmax(bounds.groupBarWidth, 0);
				} else {
					deltaY = (bounds.groupBarWidth - iHeight) / 2;
				}

				var iDeltaVal = 0;
				if (width < 11)
					iDeltaVal = 0;
				else if (width < 30)
					iDeltaVal = ((width - 10) / 2);
				else
					iDeltaVal = 10;

				var sPath = "M " + xPos + " " + (yPos + (iHeight / 2) + deltaY) + " L " + (xPos + iDeltaVal) + " " + (yPos + deltaY) + " L " + ((xPos + width) - iDeltaVal) + " " + (yPos + deltaY)
						+ " L " + (xPos + width) + " " + (yPos + (iHeight / 2) + deltaY) + " L " + ((xPos + width) - iDeltaVal) + " " + (yPos + iHeight + deltaY) + " L " + (xPos + iDeltaVal) + " "
						+ (yPos + iHeight + deltaY) + " z";

				Ext.apply(barAttr, {
					path : sPath,
					x : xPos,
					y : yPos + deltaY -1
				});
			}
			if (height < 0) {
				if (column) {
					barAttr.y = top;
					barAttr.height = mabs(height);
				} else {
					barAttr.x = top + height;
					barAttr.width = mabs(height);
				}
			}
			if (stacked) {
				if (height < 0) {
					top += height * (column ? -1 : 1);
				} else {
					bottom += height * (column ? -1 : 1);
				}
				totalDim += mabs(height);
				if (height < 0) {
					totalNegDim += mabs(height);
				}
			}
			barAttr.x = Math.floor(barAttr.x) + 1;
			floorY = Math.floor(barAttr.y);
			if (Ext.isIE8m && barAttr.y > floorY) {
				floorY--;
			}
			barAttr.y = floorY;
			barAttr.width = Math.floor(barAttr.width);
			barAttr.height = Math.floor(barAttr.height);
			items.push({
				series : me,
				yField : yFields[j],
				storeItem : record,
				value : [ record.get(me.xField), yValue ],
				attr : barAttr,
				point : column ? [ barAttr.x + barAttr.width / 2, yValue >= 0 ? barAttr.y : barAttr.y + barAttr.height ] : [ yValue >= 0 ? barAttr.x + barAttr.width : barAttr.x,
						barAttr.y + barAttr.height / 2 ]
			});
			// When resizing, reset before animating
			if (animate && chart.resizing) {
				attrs = column ? {
					x : barAttr.x,
					y : bounds.zero,
					width : barAttr.width,
					height : 0
				} : {
					x : bounds.zero,
					y : barAttr.y,
					width : 0,
					height : barAttr.height
				};
				if (enableShadows && (stacked && !hasShadow || !stacked)) {
					hasShadow = true;
					// update shadows
					for (shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
						shadow = shadowGroups[shadowIndex].getAt(stacked ? i : (i * barsLen + j));
						if (shadow) {
							shadow.setAttributes(attrs, true);
						}
					}
				}
				// update sprite position and width/height
				sprite = group.getAt(i * barsLen + j);
				if (sprite) {
					sprite.setAttributes(attrs, true);
				}
			}
			counter++;
		}
		if (stacked && items.length) {
			items[i * counter].totalDim = totalDim;
			items[i * counter].totalNegDim = totalNegDim;
			items[i * counter].totalPositiveValues = totalPositiveValues;
			items[i * counter].totalNegativeValues = totalNegativeValues;
		}
	}
	if (stacked && counter == 0) {
		// Remove ghost shadow ref: EXTJSIV-5982
		for (i = 0, total = data.length; i < total; i++) {
			for (shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
				shadow = shadowGroups[shadowIndex].getAt(i);
				if (shadow) {
					shadow.hide(true);
				}
			}
		}
	}
};

Ext.chart.axis.Axis.prototype.calcEnds = function() {
	var me = this, range = me.getRange(), min = range.min, max = range.max, steps, prettyNumbers, out, changedRange;

	steps = (Ext.isNumber(me.majorTickSteps) ? me.majorTickSteps + 1 : me.steps);
	prettyNumbers = !(Ext.isNumber(me.maximum) && Ext.isNumber(me.minimum) && Ext.isNumber(me.majorTickSteps) && me.majorTickSteps > 0);

	out = Ext.draw.Draw.snapEnds(min, max, steps, prettyNumbers);

	if (Ext.isNumber(me.maximum)) {
		out.to = me.maximum;
		changedRange = true;
	}
	if (Ext.isNumber(me.minimum)) {
		out.from = me.minimum;
		changedRange = true;
	}
	if (me.adjustMaximumByMajorUnit) {
		out.to = Math.ceil(out.to / out.step) * out.step;
		changedRange = true;
	}
	if (me.adjustMinimumByMajorUnit) {
		out.from = Math.floor(out.from / out.step) * out.step;
		changedRange = true;
	}

	if (changedRange) {
		out.steps = Math.ceil((out.to - out.from) / out.step);
	}

	me.prevMin = (min == max ? 0 : min);
	me.prevMax = max;
	out.to = 7 * 24 * 60 * 60 - 1;
	out.step = 24 * 60 * 60;
	return out;
};